//------------------------------------------------------------------------------
// CERN BE-CO-HT
// White Rabbit Trigger Distribution (WRTD)
// https://ohwr.org/projects/wrtd
//------------------------------------------------------------------------------
//
// unit name:   WrtdTstamp
//
// description: A collection of definitions and types used by the WrtdDev class.
//
// These are mostly copy-pasted from the WRTD library.
//
//------------------------------------------------------------------------------
// Copyright CERN 2019
//------------------------------------------------------------------------------
// Copyright and related rights are licensed under the Solderpad Hardware
// License, Version 2.0 (the "License"); you may not use this file except
// in compliance with the License. You may obtain a copy of the License at
// http://solderpad.org/licenses/SHL-2.0.
// Unless required by applicable law or agreed to in writing, software,
// hardware and materials distributed under this License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
// or implied. See the License for the specific language governing permissions
// and limitations under the License.
//------------------------------------------------------------------------------

`ifndef __WRTD_DEFINE_INCLUDED
 `define __WRTD_DEFINE_INCLUDED

 `define WRTD_MAX_CPUS 4
 `define WRTD_MAX_DEVS 4

 `define WRTD_ID_LEN 16

 `define WRTD_IO_MSG_WORD_SIZE  3
 `define WRTD_CFG_MSG_WORD_SIZE 5
 `define WRTD_ROOT_WORD_SIZE    13
 `define WRTD_RULE_WORD_SIZE    40
 `define WRTD_ALRM_WORD_SIZE    15

enum {
      WRTD_ACTION_GET_CONFIG,
      WRTD_ACTION_READW,
      WRTD_ACTION_WRITEW
} wrtd_trtl_actions;

enum {
      WRTD_DIR_INPUT,
      WRTD_DIR_OUTPUT
} wrtd_dir;

class WrtdTstamp;
   protected string   name;
   protected uint32_t seconds;
   protected uint32_t ns;
   protected uint32_t frac;

   function new ( uint32_t seconds = 0,
		  uint32_t ns      = 0,
		  uint32_t frac    = 0,
		  string name      = "");
      this.name = name;
      set ( seconds, ns, frac );
   endfunction // new

   task mdisplay ( string str );
      string tmp;
      if (this.name == "")
	tmp = $sformatf("<%t> %s", $realtime, str);
      else
	tmp = $sformatf("[%s] <%t> %s", this.name, $realtime, str);
      $display (tmp);
   endtask // mdisplay

   function void set ( uint32_t seconds, uint32_t ns, uint32_t frac );
      this.seconds = seconds;
      this.ns      = ns;
      this.frac    = frac;
   endfunction // set

   function void zero ( );
      set ( 0, 0, 0);
   endfunction // zero

endclass // WrtdTstamp

typedef struct {
   uint32_t addr;
   uint32_t nbr_alarms;
   uint32_t alarms_addr;
   uint32_t nbr_rules;
   uint32_t rules_addr;
   uint32_t nbr_devs;
   uint32_t devs_addr[4];
} wrtd_root;



`endif //  `ifndef __WRTD_DEFINE_INCLUDED
