//------------------------------------------------------------------------------
// CERN BE-CO-HT
// White Rabbit Trigger Distribution (WRTD)
// https://ohwr.org/projects/wrtd
//------------------------------------------------------------------------------
//
// unit name:   WrtdRule, WrtdRuleCollection
//
// description: A SystemVerilog Class for a WRTD "rule" repeated capability
//
//------------------------------------------------------------------------------
// Copyright CERN 2019
//------------------------------------------------------------------------------
// Copyright and related rights are licensed under the Solderpad Hardware
// License, Version 2.0 (the "License"); you may not use this file except
// in compliance with the License. You may obtain a copy of the License at
// http://solderpad.org/licenses/SHL-2.0.
// Unless required by applicable law or agreed to in writing, software,
// hardware and materials distributed under this License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
// or implied. See the License for the specific language governing permissions
// and limitations under the License.
//------------------------------------------------------------------------------

`ifndef __WRTD_RULE_INCLUDED
 `define __WRTD_RULE_INCLUDED

 `include "wrtd_rep_cap.svh"

class WrtdRule extends WrtdRepCap;
   protected string     src;
   protected string     dst;
   protected WrtdTstamp delay;
   protected int        send_late;

   function new ( int core, int index, string name = "" );
      super.new ( core, index, name );
      this.delay = new ();
      clear();
   endfunction // new

   function void clear ( );
      super.clear();
      this.delay.zero();
      this.src        = "";
      this.dst        = "";
      this.send_late  = 1;
   endfunction // clear

   function repcap_data data_pack ( );
   endfunction // data_pack

   function void data_unpack ( repcap_data data );
   endfunction // data_unpack

   function string get_src ( );
      return this.src;
   endfunction // get_src

   function void set ( WrtdTstamp delay, string src, string dst );
      if ( delay != null )
	this.delay = delay;
      else
	this.delay.zero();
      this.src   = src;
      this.dst   = dst;
   endfunction // set

endclass //WrtdRule

class WrtdRuleCollection extends WrtdRepCapCollection;
   protected WrtdRule collection[];

   function new ( int size, string name );
      super.new ( size, name );
   endfunction // new

   function int set ( string rep_cap_id, WrtdTstamp delay,
		      string src, string dst );
      int idx;

      idx = find ( rep_cap_id );

      if ( idx == -1 )
	$error ( "%s repeated capability ID cannot be set because it does not exist", rep_cap_id );

      if ( this.collection[idx].is_enabled() )
	$error ( "%s repeated capability ID cannot be set because it is enabled", rep_cap_id );

      this.collection[idx].set ( delay, src, dst );

      return idx;
   endfunction // set

   function string get_src ( int idx );
      return this.collection[idx].get_src();
   endfunction // get_src

endclass // WrtdRuleCollection

`endif //  `ifndef __WRTD_RULE_INCLUDED
