/*
 * Copyright (C) 2013-2018 CERN (www.cern.ch)
 * Author: Federico Vaga <federico.vaga@cern.ch>
 * Author: Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
 *
 * SPDX-License-Identifier: LGPL-3.0-or-later
 */


#include "mockturtle-rt.h"
#include <mockturtle-framework.h>
#include "wrtd-common.h"
#include "fmc_adc_alt_trigout.h"

#define NBR_CPUS 1
#define CPU_IDX 0

#define NBR_LOCAL_INPUTS 5
#define NBR_LOCAL_OUTPUTS 1
#define NBR_RULES 16
#define NBR_DEVICES 2
#define NBR_ALARMS 1
/* Addresses comes from cpu0 crossbar.  */
#define DEVICES_IO_ADDR { TRTL_ADDR_DP_BASE + 0x0000, TRTL_ADDR_DP_BASE + 0x1000, 0, 0}
#define APP_ID WRTD_APP_ADCIO_5CH
#define APP_VER RT_VERSION(2, 0)
#define APP_NAME "wrtd-adc"
#define WRTD_NET_TX
#define WRTD_NET_RX

#include "wrtd-rt-common.h"

#include "wrtd-adcout.c"
#include "wrtd-adcin.c"

static struct wrtd_adcin_dev adcin0 =
{
        .io_addr = 0x0000,
};

static struct wrtd_adcout_dev adcout0 =
{
        .io_addr = 0x1000,
};

static inline int wr_link_up(void)
{
	if (!adcin_wr_link_up(&adcin0))
		return 0;
        if (!adcout_wr_link_up(&adcout0))
		return 0;
	return 1;
}

static inline int wr_time_locked(void)
{
	if (!adcin_wr_time_locked(&adcin0))
		return 0;
        if (!adcout_wr_time_locked(&adcout0))
		return 0;
	return 1;
}

static inline int wr_time_ready(void)
{
	if (!adcin_wr_time_ready(&adcin0))
		return 0;
        if (!adcout_wr_time_ready(&adcout0))
		return 0;
	return 1;
}

static inline void wr_enable_lock(int enable)
{
	adcin_wr_enable_lock(&adcin0, enable);
	adcout_wr_enable_lock(&adcout0, enable);
}

static inline int wr_sync_timeout(void)
{
	int res, val;
	res = 0;

	val = adcin_wr_sync_timeout();
	res = val > res ? val : res;

	val = adcout_wr_sync_timeout();
	res = val > res ? val : res;
        return res;
}

static void wrtd_local_output(struct wrtd_event *ev, unsigned ch)
{
	adcout_local_output(&adcout0, ev, ch);
        return;
}

static int wrtd_user_init(void)
{
	adcin_init(&adcin0);
        adcout_init(&adcout0);

	wr_enable_lock(0);

	pr_debug("rt-input firmware initialized.\n\r");

	return 0;
}

static void wrtd_io(void)
{
	adcin_input(&adcin0);
        adcout_output(&adcout0);
}
