/*
 * Copyright (C) 2013-2018 CERN (www.cern.ch)
 * Author: Federico Vaga <federico.vaga@cern.ch>
 * Author: Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
 *
 * SPDX-License-Identifier: LGPL-3.0-or-later
 */

#include "mockturtle-rt.h"
#include <mockturtle-framework.h>
#include "wrtd-common.h"

#include "wrtd-fd.h"

#define NBR_CPUS 2
#define CPU_IDX 0

#define NBR_LOCAL_INPUTS 0
#define NBR_LOCAL_OUTPUTS FD_NUM_CHANNELS
#define NBR_RULES 16
#define NBR_DEVICES 1
#define NBR_ALARMS 1
#define DEVICES_IO_ADDR { TRTL_ADDR_DP_BASE, 0, 0, 0}
#define APP_ID WRTD_APP_FD_4CH
#define APP_VER RT_VERSION(2, 0)
#define APP_NAME "wrtd-fd"
#define WRTD_NET_RX 1

#include "wrtd-rt-common.h"

#include "wrtd-fd.c"

struct wrtd_fd_dev fd0 = {
        .io_addr = 0x0
};

static int wr_link_up(void)
{
        return fd_wr_link_up(&fd0);
}

static int wr_time_locked(void)
{
        return fd_wr_time_locked(&fd0);
}

static int wr_time_ready(void)
{
        return fd_wr_time_ready(&fd0);
}

static void wr_enable_lock(int enable)
{
        fd_wr_enable_lock(&fd0, enable);
}

static inline int wr_sync_timeout(void)
{
        return fd_wr_sync_timeout();
}

static void wrtd_local_output(struct wrtd_event *ev, unsigned ch)
{
        fd_local_output(&fd0, ev, ch);
}

#ifdef SIMULATION
static int wrtd_o_sim_init(void)
{
	int i;

	for (i = 0; i < FD_NUM_CHANNELS; i++) {
		int hidx;
		wrtd_out_channels[i].config.log_level = WRTD_LOG_ALL;
		wrtd_out_channels[i].config.state = OUT_ST_ARMED;
		wrtd_out_channels[i].config.flags =
			WRTD_ENABLED | WRTD_ARMED | WRTD_TRIGGER_ASSIGNED;

		triggers[i].id.system = 0;
		triggers[i].id.source_port = i;
		triggers[i].id.trigger = i;
		triggers[i].flags = ENTRY_FLAG_VALID;
		triggers[i].ocfg[i].delay_cycles = 100 * 125; // 1 cycle is 8ns
		triggers[i].ocfg[i].delay_frac = 0;
		triggers[i].ocfg[i].state = HASH_ENT_DIRECT;

		hidx = wrtd_out_hash_table_free(&triggers[i].id);
		tlist_count++;
		ht[hidx] = &triggers[i];
	}

	return 0;
}

#endif

/**
 * Initialize data structures, RT application and variables
 */
static int wrtd_user_init(void)
{
	if (!fd_wr_present(&fd0)) {
		pr_error("WhiteRabbit not found\n\r");
		return -EINVAL;
	}

	wr_enable_lock(0);

	/* Channels */
        wrtd_fd_data_init(&fd0);

#ifdef SIMULATION
	/* Skip WR sync and automatically generate some events when simulating */
        wrtd_o_sim_init();
        fd_sim_init();
#else
        fd_init(&fd0);
#endif

	pr_debug("rt-output firmware initialized.\n\r");

	return 0;
}

static void wrtd_io(void)
{
        fd_outputs(&fd0);
}
