#include <stdarg.h>
#include <stddef.h>
#include <stdlib.h>
#include <assert.h>
#include <errno.h>
#include "libmockturtle.h"
#include "libwrtd.h"
#include "libwrtd-private.h"
#include "hw/fmctdc-direct.h"
#include "fmc_adc_alt_trigout.h"

/* FIXME: duplicated macro.  */
#define BASE_DP_TDC_DIRECT  0x8000

static enum wrtd_status wrtd_none_configure(struct wrtd_dev *wrtd,
                                            unsigned cpu, unsigned dev,
                                            unsigned int ch_mask)
{
        return WRTD_SUCCESS;
}

static enum wrtd_status wrtd_tdc_configure(struct wrtd_dev *wrtd,
                                           unsigned cpu, unsigned dev,
                                           unsigned int ch_mask)
{
        enum wrtd_status status;
        uint32_t val;
        uint32_t addr;

        assert((ch_mask & 0x1f) == ch_mask);

        /* Just set the enable bits.  */
        val = ch_mask;
        addr = wrtd->roots[cpu].devices_io_addr[dev]
                + BASE_DP_TDC_DIRECT + DR_REG_CHAN_ENABLE;

        printf("wrtd: tdc write 0x%08x at 0x%08x on cpu#%u\n", val, addr, cpu);
        status = wrtd_msg_writew(wrtd, cpu, addr, 1, &val);
        return status;
}

static enum wrtd_status wrtd_adcin_configure(struct wrtd_dev *wrtd,
                                             unsigned cpu, unsigned dev,
                                             unsigned int ch_mask)
{
        enum wrtd_status status;
        uint32_t val;
        uint32_t addr;

        assert((ch_mask & 0x1f) == ch_mask);

        /* Just set the enable bits.  */
        val = (ch_mask & 0x0f) | ((ch_mask & 0x10) << 4);
        addr = wrtd->roots[cpu].devices_io_addr[dev] + ALT_TRIGOUT_ENABLE;

        printf("wrtd: adcin write 0x%08x at 0x%08x on cpu#%u\n", val, addr, cpu);
        status = wrtd_msg_writew(wrtd, cpu, addr, 1, &val);
        return status;
}

static const char * const tdc_channels[] =
{
        "TDC1", "TDC2", "TDC3", "TDC4", "TDC5"
};

static const char * const fd_channels[] =
{
        "FD1", "FD2", "FD3", "FD4"
};

static const char * const adcin_channels[] =
{
        "ADCI1", "ADCI2", "ADCI3", "ADCI4", "ADCI5",
};

static const char * const adcout_channels[] =
{
        "ADCO1",
};


static const struct wrtd_mt_device tdc_device =
{
        .name      = "FMC-TDC 5CH",
        .dir       = wrtd_input,
        .nbr_chs   = 5,
        .channels  = tdc_channels,
        .configure = wrtd_tdc_configure
};

static const struct wrtd_mt_device fd_device =
{
        .name      = "FMC-FD 4CH",
        .dir       = wrtd_output,
        .nbr_chs   = 4,
        .channels  = fd_channels,
        .configure = wrtd_none_configure
};

static const struct wrtd_mt_device adcin_device =
{
        .name      = "FMC ADCin 5CH",
        .dir       = wrtd_input,
        .nbr_chs   = 5,
        .channels  = adcin_channels,
        .configure = wrtd_adcin_configure
};

static const struct wrtd_mt_device adcout_device =
{
        .name      = "FMC ADCout",
        .dir       = wrtd_output,
        .nbr_chs   = 1,
        .channels  = adcout_channels,
        .configure = wrtd_none_configure
};


static const struct wrtd_mt_config configs[] =
{
        /* TDC + FD board.  */
        {
                .mt_app_id = 0x115790de,
                .nbr_cpus  = 2,
                .tx_cpu    = 0,
                .rx_cpu    = 1,
                .cpus = {
                        {
                                WRTD_APP_TDC_5CH,
                                1,
                                {
                                        &tdc_device
                                }

                        },
                        {
                                WRTD_APP_FD_4CH,
                                1,
                                {
                                        &fd_device
                                }
                        }
                }
        },
        /* ADC board.  */
        {
                .mt_app_id = 0x115790d1,
                .nbr_cpus  = 1,
                .tx_cpu    = 0,
                .rx_cpu    = 0,
                .cpus = {
                        {
                                WRTD_APP_ADCIO_5CH,
                                2,
                                {
                                        &adcin_device,
                                        &adcout_device
                                }

                        }
                }
        }

};

enum wrtd_status wrtd_find_config(struct wrtd_dev *dev,
                                  const struct wrtd_mt_config **config)
{
        const struct trtl_config_rom *cfgrom;
        int i;

        cfgrom = trtl_config_get(dev->trtl);
        for (i = 0; i < sizeof(configs) / sizeof(configs[0]); i++)
                if (configs[i].mt_app_id == cfgrom->app_id
                    && configs[i].nbr_cpus == cfgrom->n_cpu) {
                        *config = &configs[i];
                        return WRTD_SUCCESS;
                }
        return wrtd_return_error(dev, WRTD_ERROR_RESOURCE_UNKNOWN,
                                 "mt app_id 0x%x n_cpu=%d unknown",
                                 cfgrom->app_id, cfgrom->n_cpu);
}
